@@ -3,7 +3,9 @@ import os
 import sys
 import json
 import time
+import uuid
 import logging
+import requests
 import threading
 import logging.config
 import numpy as np
@@ -20,14 +22,6 @@ from anytree import AnyNode, RenderTree, DoubleStyle
 def run_from_unittest():
     return 'unittest' in sys.services
 
-
-known_host = {
-    'ergo': ['/dev/cu.usbserial-DN2AAOVK', '/dev/cu.usbserial-DN2YEFLN'],
-    'handy': ['/dev/cu.usbserial-DN2X236E'],
-    'eddy': ['pi-gate.local'],
-}
-
-
 class contList(list):
     def __repr__(self):
         s = '-------------------------------------------------\n'
@@ -86,28 +80,20 @@ class nodeList(list):
             prefill = fill
         return s
 
+
 class Device(object):
     _heartbeat_timeout = 5  # in sec.
     _max_alias_length = 15
     _base_log_conf = os.path.join(os.path.dirname(__file__),
                                   'logging_conf.json')
-
-    @classmethod
-    def discover(cls):
-        hosts = discover_hosts()
-
-        possibilities = {
-            k: [h for h in v if h in hosts]
-            for k, v in known_host.items()
-        }
-
-        return possibilities
+    _freedomLink = None
 
     def __init__(self, host,
                  IO=None,
                  log_conf=_base_log_conf,
                  test_mode=False,
                  background_task=True,
+                 telemetry=True,
                  *args, **kwargs):
         if IO is not None:
             self._io = IO(host=host, *args, **kwargs)
@@ -120,6 +106,7 @@ class Device(object):
                 config = json.load(f)
             logging.config.dictConfig(config)
 
+        self.telemetry = telemetry
         self.logger = logging.getLogger(__name__)
         self.logger.info('Connected to "{}".'.format(host))
 
@@ -139,43 +126,21 @@ class Device(object):
             self._poll_bg = threading.Thread(target=self._poll_and_up)
             self._poll_bg.daemon = True
             self._poll_bg.start()
-        self._baudrate = 1000000
 
     def close(self):
         self._running = False
-        self._poll_bg.join()
+
+        if hasattr(self, "_poll_bg"):
+            self._poll_bg.join(timeout=2.0)
+
+            if self._poll_bg.is_alive():
+                # _poll_bg didn't terminate within the timeout
+                print("Warning: device closed on timeout, background thread is still running.")
         self._io.close()
 
-    @property
-    def baudrate(self):
-        return self._baudrate
-
-    @baudrate.setter
-    def baudrate(self, baudrate):
-        self._send({'baudrate': baudrate})
-        self._baudrate = baudrate
-        time.sleep(0.01)
-
-    def benchmark(self, target_id, data, repetition):
-        data = np.array(data, dtype=np.uint8)
-        self._bench_settings = {'benchmark': {'target': target_id, 'repetitions': repetition, 'data': [len(data)]}}
-        self._bench_Data = data.tobytes()
-        self._write( json.dumps(self._bench_settings).encode() + '\r'.encode() + self._bench_Data)
-
-        state = self._poll_once()
-        startTime = time.time()
-        retry = 0
-        while ('benchmark' not in state):
-            state = self._poll_once()
-            if (time.time()-startTime > 30):
-                self._write( json.dumps(self._bench_settings).encode() + '\r'.encode() + self._bench_Data)
-                retry = retry+1
-                if (retry == 3):
-                    return (0, 100)
-                startTime = time.time()
-
-        #self._pause = False
-        return (state['benchmark']['data_rate'], state['benchmark']['fail_rate'])
+    def link_to_freedomrobotics(self):
+        from .integration.freedomRobotics import FreedomLink
+        self._freedomLink = FreedomLink(self)
 
     def pause(self):
         self._pause = True
@@ -186,16 +151,23 @@ class Device(object):
 
     def _setup(self):
         self.logger.info('Sending detection signal.')
+        self._send({})
+        time.sleep(0.01)
         self._send({'detection': {}})
         self.logger.info('Waiting for routing table...')
         startTime = time.time()
         state = self._poll_once()
+        retry = 0
         while ('routing_table' not in state):
             if ('route_table' in state):
                 self.logger.info("Watch out the Luos revision you are using on your board is too old to work with this revision on pyluos.\n Please consider updating Luos on your boards")
                 return
             state = self._poll_once()
             if (time.time()-startTime > 1):
+                retry = retry +1
+                if retry > 5:
+                    # detection is not working
+                    sys.exit("Detection failed.")
                 self._send({'detection': {}})
                 startTime = time.time()
         # Create nodes
@@ -234,6 +206,25 @@ class Device(object):
         self._cmd_data = []
         self._binary = []
 
+        if (self.telemetry == True):
+            from pyluos.version import version
+            self.logger.info('Sending telemetry...')
+            luos_telemetry = {"telemetry_type": "pyluos",
+                  "mac": hex(uuid.getnode()),
+                  "system": sys.platform,
+                  "unix_time": int(time.time()),
+                  "pyluos_rev": version,
+                  "routing_table":state['routing_table']}
+            try:
+                r = requests.post("https://monorepo-services.vercel.app/api/telemetry",
+                           data=luos_telemetry)
+                if not r:
+                    print("Telemetry request failed : error " + str(r.status_code))
+            except:
+                print("Telemetry request failed.")
+        else:
+            self.logger.info("Telemetry disabled, please consider enabling it by removing the 'telemetry=False' argument of your Device creation.")
+
         # We push our current state to make sure that
         # both our model and the hardware are synced.
         self._push_once()
@@ -249,38 +240,47 @@ class Device(object):
     # Poll state from hardware.
     def _poll_once(self):
         self._state = self._io.read()
-        self._state['timestamp'] = time.time()
-        return self._state
+        if self._state != []:
+            self._state['timestamp'] = time.time()
+            return self._state
+        return []
 
     def _poll_and_up(self):
         while self._running:
             if not self._pause :
                 state = self._poll_once()
-                self._update(state)
-                self._push_once()
+                if self._state != []:
+                    self._update(state)
+                    self._push_once()
             else :
                 time.sleep(0.1)
 
     # Update our model with the new state.
     def _update(self, new_state):
-        if 'dead_service' in new_state :
+        if 'dead_service' in new_state.keys() :
             #we have lost a service put a flag on this service
             alias = new_state['dead_service']
             if hasattr(self, alias):
                 getattr(self, alias)._kill()
-        if 'assert' in new_state :
+            if (self._freedomLink != None):
+                self._freedomLink._kill(alias)
+        if 'assert' in new_state.keys() :
             # A node assert, print assert informations
             if (('node_id' in new_state['assert']) and ('file' in new_state['assert']) and ('line' in new_state['assert'])):
                 s = "************************* ASSERT *************************\n"
                 s += "*  Node " + str(new_state['assert']['node_id']) + " assert in file " + new_state['assert']['file'] + " line " + str(new_state['assert']['line'])
                 s += "\n**********************************************************"
                 print (s)
-        if 'services' not in new_state:
+            if (self._freedomLink != None):
+                self._freedomLink._assert(alias)
+        if 'services' not in new_state.keys():
             return
 
         for alias, mod in new_state['services'].items():
             if hasattr(self, alias):
                 getattr(self, alias)._update(mod)
+            if (self._freedomLink != None):
+                self._freedomLink._update(alias, mod)
 
         self._last_update = time.time()
 
@@ -300,7 +300,7 @@ class Device(object):
                 self._cmd = defaultdict(lambda: defaultdict(lambda: None))
             for cmd, binary in zip(self._cmd_data, self._binary):
                 time.sleep(0.01)
-                self._write( json.dumps({'services': cmd}).encode() + '\r'.encode() + binary)
+                self._write( json.dumps({'services': cmd}).encode() + '\n'.encode() + binary)
 
             self._cmd_data = []
             self._binary = []
