@@ -36,7 +36,8 @@ class IOHandler(object):
     def loads(self, data):
         if type(data) == bytes:
             data = data.decode()
-        return json.loads(data)
+            return json.loads(data)
+        return []
 
     def dumps(self, msg):
         return json.dumps(msg).encode()
@@ -51,7 +52,7 @@ IOs = [Serial, Ws]
 def io_from_host(host, *args, **kwargs):
     for cls in IOs:
         if cls.is_host_compatible(host):
-            return cls(host=host, *args, **kwargs)
+            return cls(host=host)
 
     raise ValueError('No corresponding IO found (among {}).'.format(discover_hosts))
 
