@@ -297,7 +297,10 @@ class ServoMotor(Service):
         if (self._config[ServoMotor._MODE_ANGULAR_SPEED] != True):
             print("rotation speed mode could be not enabled in the service please use 'device.service.rot_speed_mode = True' to enable it")
         self._target_rot_speed = s
-        self._push_value("target_rot_speed", s)
+        if hasattr(s, "__len__"):
+            self._push_data('target_rot_speed', [len(s) * 4], np.array(s, dtype=np.float32)) # multiplying by the size of float32
+        else :
+            self._push_value("target_rot_speed", s)
 
     @property
     def rot_speed_mode(self):
@@ -496,17 +499,17 @@ class ServoMotor(Service):
 
     def _update(self, new_state):
         Service._update(self, new_state)
-        if 'rot_position' in new_state:
+        if 'rot_position' in new_state.keys():
             self._rot_position = new_state['rot_position']
-        if 'rot_speed' in new_state:
+        if 'rot_speed' in new_state.keys():
             self._rot_speed = new_state['rot_speed']
-        if 'trans_position' in new_state:
+        if 'trans_position' in new_state.keys():
             self._trans_position = new_state['trans_position']
-        if 'trans_speed' in new_state:
+        if 'trans_speed' in new_state.keys():
             self._trans_speed = new_state['trans_speed']
-        if 'current' in new_state:
+        if 'current' in new_state.keys():
             self._current = new_state['current']
-        if 'temperature' in new_state:
+        if 'temperature' in new_state.keys():
             self._temperature = new_state['temperature']
 
     def control(self):
